@@ -25,3 +25,18 @@ class OWLObjectRenderer(metaclass=ABCMeta):
             String rendition of OWL object
         """
         pass
+
+
+class OWLObjectParser(metaclass=ABCMeta):
+    """Abstract class with a parse method to parse a string to an OWL Object"""
+    @abstractmethod
+    def parse_expression(self, expression_str: str) -> OWLObject:
+        """Parse a string to an OWL Object
+
+        Args:
+            expression_str: string
+
+        Returns:
+            The OWL Object which is represented by the string
+        """
+        pass
