@@ -21,7 +21,9 @@ MOVE(OWLObject, OWLAnnotationObject, OWLAnnotationSubject, OWLAnnotationValue, H
 
 _T = TypeVar('_T')  #:
 _C = TypeVar('_C', bound='OWLObject')  #:
-Literals = Union['OWLLiteral', int, float, bool, Timedelta, datetime, date]  #:
+_P = TypeVar('_P', bound='OWLPropertyExpression')  #:
+_R = TypeVar('_R', bound='OWLPropertyRange')  #:
+Literals = Union['OWLLiteral', int, float, bool, Timedelta, datetime, date, str]  #:
 
 
 class HasIndex(Protocol):
@@ -778,7 +780,7 @@ class OWLObjectExactCardinality(OWLObjectCardinalityRestriction):
         Returns:
             The semantically equivalent but structurally simpler form (= 1 R C) = >= 1 R C and <= 1 R C
         """
-        args = self.get_property(), self.get_cardinality(), self.get_filler()
+        args = self.get_cardinality(), self.get_property(), self.get_filler()
         return OWLObjectIntersectionOf((OWLObjectMinCardinality(*args), OWLObjectMaxCardinality(*args)))
 
 
@@ -1002,9 +1004,23 @@ class OWLAxiom(OWLObject, metaclass=ABCMeta):
     An OWL ontology contains a set of axioms. These axioms can be annotation axioms, declaration axioms, imports axioms
     or logical axioms
     """
-    __slots__ = ()
+    __slots__ = '_annotations'
+
+    _annotations: Optional[List['OWLAnnotation']]
+
+    def __init__(self, annotations: Optional[Iterable['OWLAnnotation']] = None):
+        if annotations is not None:
+            self._annotations = list(annotations)
+
+    def annotations(self) -> Optional[List['OWLAnnotation']]:
+        return self._annotations
+
+    def is_annotated(self) -> bool:
+        return self._annotations is not None and len(self._annotations) > 0
+
+    def is_logical_axiom(self) -> bool:
+        return False
     # TODO: XXX
-    pass
 
 
 class OWLDatatype(OWLEntity, OWLDataRange):
@@ -1108,26 +1124,46 @@ class OWLLiteral(OWLAnnotationValue, metaclass=ABCMeta):
 
     type_index: Final = 4008
 
-    def __new__(cls, value):
+    def __new__(cls, value, type_: Optional[OWLDatatype] = None):
         """Convenience method that obtains a literal
 
         Args:
             value: The value of the literal
+            type_: the datatype of the literal
         """
+        if type_ is not None:
+            if type_ == BooleanOWLDatatype:
+                return super().__new__(_OWLLiteralImplBoolean)
+            elif type_ == IntegerOWLDatatype:
+                return super().__new__(_OWLLiteralImplInteger)
+            elif type_ == DoubleOWLDatatype:
+                return super().__new__(_OWLLiteralImplDouble)
+            elif type_ == StringOWLDatatype:
+                return super().__new__(_OWLLiteralImplString)
+            elif type_ == DateOWLDatatype:
+                return super().__new__(_OWLLiteralImplDate)
+            elif type_ == DateTimeOWLDatatype:
+                return super().__new__(_OWLLiteralImplDateTime)
+            elif type_ == DurationOWLDatatype:
+                return super().__new__(_OWLLiteralImplDuration)
+            else:
+                return super().__new__(_OWLLiteralImpl)
         if isinstance(value, bool):
             return super().__new__(_OWLLiteralImplBoolean)
         elif isinstance(value, int):
             return super().__new__(_OWLLiteralImplInteger)
         elif isinstance(value, float):
             return super().__new__(_OWLLiteralImplDouble)
-        elif isinstance(value, date):
-            return super().__new__(_OWLLiteralImplDate)
+        elif isinstance(value, str):
+            return super().__new__(_OWLLiteralImplString)
         elif isinstance(value, datetime):
             return super().__new__(_OWLLiteralImplDateTime)
+        elif isinstance(value, date):
+            return super().__new__(_OWLLiteralImplDate)
         elif isinstance(value, Timedelta):
             return super().__new__(_OWLLiteralImplDuration)
         # TODO XXX
-        raise NotImplementedError
+        raise NotImplementedError(value)
 
     def get_literal(self) -> str:
         """Gets the lexical value of this literal. Note that the language tag is not included.
@@ -1176,6 +1212,19 @@ class OWLLiteral(OWLAnnotationValue, metaclass=ABCMeta):
         """
         raise ValueError
 
+    def is_string(self) -> bool:
+        """Whether this literal is typed as string"""
+        return False
+
+    def parse_string(self) -> str:
+        """Parses the lexical value of this literal into a string. The lexical value of this literal should be in the
+        lexical space of the string datatype ("http://www.w3.org/2001/XMLSchema#string").
+
+        Returns:
+            A string value that is represented by this literal.
+        """
+        raise ValueError
+
     def is_date(self) -> bool:
         """Whether this literal is typed as date"""
         return False
@@ -1243,8 +1292,10 @@ class _OWLLiteralImplDouble(OWLLiteral):
 
     _v: float
 
-    def __init__(self, value):
-        assert isinstance(value, float)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == DoubleOWLDatatype
+        if not isinstance(value, float):
+            value = float(value)
         self._v = value
 
     def __eq__(self, other):
@@ -1282,8 +1333,10 @@ class _OWLLiteralImplInteger(OWLLiteral):
 
     _v: int
 
-    def __init__(self, value):
-        assert isinstance(value, int)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == IntegerOWLDatatype
+        if not isinstance(value, int):
+            value = int(value)
         self._v = value
 
     def __eq__(self, other):
@@ -1320,8 +1373,11 @@ class _OWLLiteralImplBoolean(OWLLiteral):
 
     _v: bool
 
-    def __init__(self, value):
-        assert isinstance(value, bool)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == BooleanOWLDatatype
+        if not isinstance(value, bool):
+            from distutils.util import strtobool
+            value = bool(strtobool(value))
         self._v = value
 
     def __eq__(self, other):
@@ -1348,14 +1404,60 @@ class _OWLLiteralImplBoolean(OWLLiteral):
         return BooleanOWLDatatype
 
 
+@total_ordering
+class _OWLLiteralImplString(OWLLiteral):
+    __slots__ = '_v'
+
+    _v: str
+
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == StringOWLDatatype
+        if not isinstance(value, str):
+            value = str(value)
+        self._v = value
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return self._v == other._v
+        return NotImplemented
+
+    def __lt__(self, other):
+        if type(other) is type(self):
+            return self._v < other._v
+        return NotImplemented
+
+    def __len__(self):
+        return len(self._v)
+
+    def __hash__(self):
+        return hash(self._v)
+
+    def __repr__(self):
+        return f'OWLLiteral({self._v})'
+
+    def is_string(self) -> bool:
+        return True
+
+    def parse_string(self) -> str:
+        # documented in parent
+        return self._v
+
+    # noinspection PyMethodMayBeStatic
+    def get_datatype(self) -> OWLDatatype:
+        # documented in parent
+        return StringOWLDatatype
+
+
 @total_ordering
 class _OWLLiteralImplDate(OWLLiteral):
     __slots__ = '_v'
 
     _v: date
 
-    def __init__(self, value):
-        assert isinstance(value, date)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == DateOWLDatatype
+        if not isinstance(value, date):
+            value = date.fromisoformat(value)
         self._v = value
 
     def __eq__(self, other):
@@ -1393,8 +1495,11 @@ class _OWLLiteralImplDateTime(OWLLiteral):
 
     _v: datetime
 
-    def __init__(self, value):
-        assert isinstance(value, datetime)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == DateTimeOWLDatatype
+        if not isinstance(value, datetime):
+            value = value.replace("Z", "+00:00") if isinstance(value, str) and value[-1] == "Z" else value
+            value = datetime.fromisoformat(value)
         self._v = value
 
     def __eq__(self, other):
@@ -1432,10 +1537,15 @@ class _OWLLiteralImplDuration(OWLLiteral):
 
     _v: Timedelta
 
-    def __init__(self, value):
-        assert isinstance(value, Timedelta)
+    def __init__(self, value, type_=None):
+        assert type_ is None or type_ == DurationOWLDatatype
+        if not isinstance(value, Timedelta):
+            value = Timedelta(value)
         self._v = value
 
+    def get_literal(self) -> str:
+        return self._v.isoformat()
+
     def __eq__(self, other):
         if type(other) is type(self):
             return self._v == other._v
@@ -1465,6 +1575,29 @@ class _OWLLiteralImplDuration(OWLLiteral):
         return DurationOWLDatatype
 
 
+class _OWLLiteralImpl(OWLLiteral):
+    __slots__ = '_v', '_datatype'
+
+    def __init__(self, v, type_: OWLDatatype):
+        assert isinstance(type_, OWLDatatype)
+        self._v = v
+        self._datatype = type_
+
+    def get_datatype(self) -> OWLDatatype:
+        return self._datatype
+
+    def __eq__(self, other):
+        if type(other) is type(self) and other.get_datatype() == self.get_datatype():
+            return self._v == other._v
+        return NotImplemented
+
+    def __hash__(self):
+        return hash((self._v, self._datatype))
+
+    def __repr__(self):
+        return f'OWLLiteral({repr(self._v)}, {self._datatype})'
+
+
 class OWLQuantifiedDataRestriction(OWLQuantifiedRestriction[OWLDataRange],
                                    OWLDataRestriction, metaclass=ABCMeta):
     """A quantified data restriction."""
@@ -1632,7 +1765,7 @@ class OWLDataExactCardinality(OWLDataCardinalityRestriction):
         Returns:
             The semantically equivalent but structurally simpler form (= 1 R D) = >= 1 R D and <= 1 R D
         """
-        args = self.get_property(), self.get_cardinality(), self.get_filler()
+        args = self.get_cardinality(), self.get_property(), self.get_filler()
         return OWLObjectIntersectionOf((OWLDataMinCardinality(*args), OWLDataMaxCardinality(*args)))
 
 
@@ -1869,7 +2002,20 @@ class OWLLogicalAxiom(OWLAxiom, metaclass=ABCMeta):
     (including imports declarations) and annotation axioms.
     """
     __slots__ = ()
-    pass
+
+    def __init__(self, annotations: Optional[Iterable['OWLAnnotation']] = None):
+        super().__init__(annotations=annotations)
+
+    def is_logical_axiom(self) -> bool:
+        return True
+
+
+class OWLPropertyAxiom(OWLLogicalAxiom, metaclass=ABCMeta):
+    """The base interface for property axioms."""
+    __slots__ = ()
+
+    def __init__(self, annotations: Optional[Iterable['OWLAnnotation']] = None):
+        super().__init__(annotations=annotations)
 
 
 class OWLClassAxiom(OWLLogicalAxiom, metaclass=ABCMeta):
@@ -1933,6 +2079,38 @@ class OWLEquivalentClassesAxiom(OWLNaryClassAxiom):
         super().__init__([cls_a, cls_b])
 
 
+class OWLSubClassOfAxiom(OWLClassAxiom):
+    """Represents an SubClassOf axiom in the OWL 2 Specification."""
+    __slots__ = '_sub_class', '_super_class'
+
+    def __init__(self, sub_class: OWLClassExpression, super_class: OWLClassExpression):
+        """Get an equivalent classes axiom with specified operands and no annotations
+
+        Args:
+            sub_class: the sub class
+            super_class: the super class
+        """
+        self._sub_class = sub_class
+        self._super_class = super_class
+
+    def get_sub_class(self) -> OWLClassExpression:
+        return self._sub_class
+
+    def get_super_class(self) -> OWLClassExpression:
+        return self._super_class
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return self._super_class == other._super_class and self._sub_class == other._sub_class
+        return NotImplemented
+
+    def __hash__(self):
+        return hash((self._super_class, self._sub_class))
+
+    def __repr__(self):
+        return f'OWLSubClassOfAxiom(sub_class={self._sub_class},super_class={self._super_class})'
+
+
 class OWLAnnotationAxiom(OWLAxiom, metaclass=ABCMeta):
     """A super interface for annotation axioms."""
     __slots__ = ()
@@ -2054,20 +2232,57 @@ class OWLAnnotationAssertionAxiom(OWLAnnotationAxiom):
         return f'OWLAnnotationAssertionAxiom({self._subject}, {self._annotation})'
 
 
-class OWLDataPropertyRangeAxiom(OWLLogicalAxiom):
-    __slots__ = '_property', '_range', '_annotations'
-    _property: OWLDataPropertyExpression
-    _range: OWLDataRange
-    _annotations: Optional[List[OWLAnnotation]]
+class OWLUnaryPropertyAxiom(Generic[_P], OWLPropertyAxiom, metaclass=ABCMeta):
+    __slots__ = '_property'
 
-    def __init__(self, property: OWLDataPropertyExpression, range: OWLDataRange,
+    _property: _P
+
+    def __init__(self, property_: _P, annotations: Optional[Iterable[OWLAnnotation]] = None):
+        self._property = property_
+        super().__init__(annotations=annotations)
+
+    def get_property(self) -> _P:
+        return self._property
+
+
+class OWLPropertyDomainAxiom(Generic[_P], OWLUnaryPropertyAxiom[_P], metaclass=ABCMeta):
+    """Represents ObjectPropertyDomain axioms in the OWL 2 specification."""
+    __slots__ = '_domain'
+
+    _domain: OWLClassExpression
+
+    def __init__(self, property_: _P, domain: OWLClassExpression,
                  annotations: Optional[Iterable[OWLAnnotation]] = None):
-        self._property = property
-        self._range = range
-        if annotations is not None:
-            self._annotations = list(annotations)
+        self._domain = domain
+        super().__init__(property_=property_, annotations=annotations)
 
-    def get_range(self) -> OWLDataRange:
+    def get_domain(self) -> OWLClassExpression:
+        return self._domain
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return self._property == other._property and self._domain == other._domain \
+                   and self._annotations == other._annotation
+        return NotImplemented
+
+    def __hash__(self):
+        return hash((self._property, self._domain, self._annotations))
+
+    def __repr__(self):
+        return f"{type(self).__name__}({repr(self._property)},{repr(self._domain)},{repr(self._annotations)})"
+
+
+class OWLPropertyRangeAxiom(Generic[_P, _R], OWLUnaryPropertyAxiom[_P], metaclass=ABCMeta):
+    """Represents ObjectPropertyRange axioms in the OWL 2 specification."""
+    __slots__ = '_range'
+
+    _range: _R
+
+    def __init__(self, property_: _P, range_: _R, annotations: Optional[Iterable[OWLAnnotation]] = None):
+        self._range = range_
+        super().__init__(property_=property_, annotations=annotations)
+
+    def get_range(self) -> _R:
         return self._range
 
     def __eq__(self, other):
@@ -2080,7 +2295,43 @@ class OWLDataPropertyRangeAxiom(OWLLogicalAxiom):
         return hash((self._property, self._range, self._annotations))
 
     def __repr__(self):
-        return f'OWLDataPropertyRangeAxiom({self._property}, {self._range}, {self._annotations})'
+        return f"{type(self).__name__}({repr(self._property)},{repr(self._range)},{repr(self._annotations)})"
+
+
+class OWLObjectPropertyDomainAxiom(OWLPropertyDomainAxiom[OWLObjectPropertyExpression]):
+    """ Represents a ObjectPropertyDomain axiom in the OWL 2 Specification."""
+    __slots__ = ()
+
+    def __init__(self, property_: OWLObjectPropertyExpression, domain: OWLClassExpression,
+                 annotations: Optional[Iterable[OWLAnnotation]] = None):
+        super().__init__(property_=property_, domain=domain, annotations=annotations)
+
+
+class OWLDataPropertyDomainAxiom(OWLPropertyDomainAxiom[OWLDataPropertyExpression]):
+    """ Represents a DataPropertyDomain axiom in the OWL 2 Specification."""
+    __slots__ = ()
+
+    def __init__(self, property_: OWLDataPropertyExpression, domain: OWLClassExpression,
+                 annotations: Optional[Iterable[OWLAnnotation]] = None):
+        super().__init__(property_=property_, domain=domain, annotations=annotations)
+
+
+class OWLObjectPropertyRangeAxiom(OWLPropertyRangeAxiom[OWLObjectPropertyExpression, OWLClassExpression]):
+    """ Represents a ObjectPropertyRange axiom in the OWL 2 Specification."""
+    __slots__ = ()
+
+    def __init__(self, property_: OWLObjectPropertyExpression, range_: OWLClassExpression,
+                 annotations: Optional[Iterable[OWLAnnotation]] = None):
+        super().__init__(property_=property_, range_=range_, annotations=annotations)
+
+
+class OWLDataPropertyRangeAxiom(OWLPropertyRangeAxiom[OWLDataPropertyExpression, OWLDataRange]):
+    """ Represents a DataPropertyRange axiom in the OWL 2 Specification."""
+    __slots__ = ()
+
+    def __init__(self, property_: OWLDataPropertyExpression, range_: OWLDataRange,
+                 annotations: Optional[Iterable[OWLAnnotation]] = None):
+        super().__init__(property_=property_, range_=range_, annotations=annotations)
 
 
 class OWLOntology(OWLObject, metaclass=ABCMeta):
@@ -2132,6 +2383,18 @@ class OWLOntology(OWLObject, metaclass=ABCMeta):
         """
         pass
 
+    @abstractmethod
+    def data_property_domain_axioms(self, property: OWLDataProperty) -> Iterable[OWLDataPropertyDomainAxiom]:
+        """Gets the OWLDataPropertyDomainAxiom objects where the property is equal to the specified property.
+
+        Args:
+            property: The property which is equal to the property of the retrieved axioms.
+
+        Returns:
+            the axioms matching the search.
+        """
+        pass
+
     @abstractmethod
     def data_property_range_axioms(self, property: OWLDataProperty) -> Iterable[OWLDataPropertyRangeAxiom]:
         """Gets the OWLDataPropertyRangeAxiom objects where the property is equal to the specified property.
@@ -2144,6 +2407,30 @@ class OWLOntology(OWLObject, metaclass=ABCMeta):
         """
         pass
 
+    @abstractmethod
+    def object_property_domain_axioms(self, property: OWLObjectProperty) -> Iterable[OWLObjectPropertyDomainAxiom]:
+        """Gets the OWLObjectPropertyDomainAxiom objects where the property is equal to the specified property.
+
+        Args:
+            property: The property which is equal to the property of the retrieved axioms.
+
+        Returns:
+            the axioms matching the search.
+        """
+        pass
+
+    @abstractmethod
+    def object_property_range_axioms(self, property: OWLObjectProperty) -> Iterable[OWLObjectPropertyRangeAxiom]:
+        """Gets the OWLObjectPropertyRangeAxiom objects where the property is equal to the specified property.
+
+        Args:
+            property: The property which is equal to the property of the retrieved axioms.
+
+        Returns:
+            the axioms matching the search.
+        """
+        pass
+
     @abstractmethod
     def get_owl_ontology_manager(self) -> _M:
         """Gets the manager that manages this ontology"""
@@ -2285,8 +2572,8 @@ class OWLReasoner(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def data_property_domains(self, pe: OWLDataProperty, direct: bool = False) -> Iterable[OWLClass]:
-        """Gets the named classes that are the direct or indirect domains of this property with respect to the imports
+    def data_property_domains(self, pe: OWLDataProperty, direct: bool = False) -> Iterable[OWLClassExpression]:
+        """Gets the class expressions that are the direct or indirect domains of this property with respect to the imports
         closure of the root ontology.
 
         Args:
@@ -2304,8 +2591,8 @@ class OWLReasoner(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def object_property_domains(self, pe: OWLObjectProperty, direct: bool = False) -> Iterable[OWLClass]:
-        """Gets the named classes that are the direct or indirect domains of this property with respect to the imports
+    def object_property_domains(self, pe: OWLObjectProperty, direct: bool = False) -> Iterable[OWLClassExpression]:
+        """Gets the class expressions that are the direct or indirect domains of this property with respect to the imports
         closure of the root ontology.
 
         Args:
@@ -2322,8 +2609,8 @@ class OWLReasoner(metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def object_property_ranges(self, pe: OWLObjectProperty, direct: bool = False) -> Iterable[OWLClass]:
-        """Gets the named classes that are the direct or indirect ranges of this property with respect to the imports
+    def object_property_ranges(self, pe: OWLObjectProperty, direct: bool = False) -> Iterable[OWLClassExpression]:
+        """Gets the class expressions that are the direct or indirect ranges of this property with respect to the imports
         closure of the root ontology.
 
         Args:
@@ -2520,10 +2807,12 @@ OWLBottomDataProperty: Final = OWLDataProperty(OWLRDFVocabulary.OWL_BOTTOM_DATA_
 DoubleOWLDatatype: Final = OWLDatatype(XSDVocabulary.DOUBLE)  #: An object representing a double datatype.
 IntegerOWLDatatype: Final = OWLDatatype(XSDVocabulary.INTEGER)  #: An object representing an integer datatype.
 BooleanOWLDatatype: Final = OWLDatatype(XSDVocabulary.BOOLEAN)  #: An object representing the boolean datatype.
+StringOWLDatatype: Final = OWLDatatype(XSDVocabulary.STRING)  #: An object representing the string datatype.
 DateOWLDatatype: Final = OWLDatatype(XSDVocabulary.DATE)  #: An object representing the date datatype.
 DateTimeOWLDatatype: Final = OWLDatatype(XSDVocabulary.DATE_TIME)  #: An object representing the dateTime datatype.
 DurationOWLDatatype: Final = OWLDatatype(XSDVocabulary.DURATION)  #: An object representing the duration datatype.
-TopDatatype: Final = OWLDatatype(OWLRDFVocabulary.RDFS_LITERAL)  #: The OWL Datatype corresponding to the top data type
+#: The OWL Datatype corresponding to the top data type
+TopOWLDatatype: Final = OWLDatatype(OWLRDFVocabulary.RDFS_LITERAL)
 
-NUMERIC_DATATYPES: Set[OWLDatatype] = {DoubleOWLDatatype, IntegerOWLDatatype}
+NUMERIC_DATATYPES: Final[Set[OWLDatatype]] = {DoubleOWLDatatype, IntegerOWLDatatype}
 TIME_DATATYPES: Final[Set[OWLDatatype]] = {DateOWLDatatype, DateTimeOWLDatatype, DurationOWLDatatype}
