@@ -14,6 +14,7 @@ from owlapy.model import OWLLiteral, OWLNaryDataRange, OWLObject, OWLClass, OWLO
     OWLFacetRestriction, OWLDatatypeRestriction, OWLDatatype, OWLDataAllValuesFrom, OWLDataComplementOf, \
     OWLDataUnionOf, OWLDataIntersectionOf, OWLDataHasValue, OWLDataOneOf, OWLDataMaxCardinality, \
     OWLDataMinCardinality, OWLDataExactCardinality
+from owlapy.vocab import OWLFacet
 
 
 _DL_SYNTAX = types.SimpleNamespace(
@@ -153,7 +154,12 @@ class DLSyntaxObjectRenderer(OWLObjectRenderer):
 
     @render.register
     def _(self, r: OWLFacetRestriction) -> str:
-        return "%s %s" % (r.get_facet().symbolic_form, r.get_facet_value().get_literal())
+        symbolic_form = r.get_facet().symbolic_form
+        if r.get_facet() == OWLFacet.MIN_INCLUSIVE:
+            symbolic_form = _DL_SYNTAX.MIN
+        elif r.get_facet() == OWLFacet.MAX_INCLUSIVE:
+            symbolic_form = _DL_SYNTAX.MAX
+        return "%s %s" % (symbolic_form, r.get_facet_value().get_literal())
 
     @render.register
     def _(self, r: OWLDatatypeRestriction) -> str:
@@ -237,24 +243,27 @@ _MAN_SYNTAX = types.SimpleNamespace(
     OR="or",
     INVERSE="inverse",
     COMMA=",",
-    SELF="self",
+    SELF="Self",
     VALUE="value",
 )
 
 
 class ManchesterOWLSyntaxOWLObjectRenderer(OWLObjectRenderer):
     """Manchester Syntax renderer for OWL Objects"""
-    __slots__ = '_sfp'
+    __slots__ = '_sfp', '_no_render_thing'
 
     _sfp: Callable[[OWLEntity], str]
 
-    def __init__(self, short_form_provider: Callable[[OWLEntity], str] = _simple_short_form_provider):
+    def __init__(self, short_form_provider: Callable[[OWLEntity], str] = _simple_short_form_provider,
+                 no_render_thing=False):
         """Create a new Manchester Syntax renderer
 
         Args:
             short_form_provider: custom short form provider
+            no_render_thing: disable manchester rendering for Thing and Nothing
         """
         self._sfp = short_form_provider
+        self._no_render_thing = no_render_thing
 
     def set_short_form_provider(self, short_form_provider: Callable[[OWLEntity], str]) -> None:
         self._sfp = short_form_provider
@@ -266,12 +275,12 @@ class ManchesterOWLSyntaxOWLObjectRenderer(OWLObjectRenderer):
 
     @render.register
     def _(self, o: OWLClass) -> str:
-        if o.is_owl_nothing():
-            return _MAN_SYNTAX.BOTTOM
-        elif o.is_owl_thing():
-            return _MAN_SYNTAX.TOP
-        else:
-            return self._sfp(o)
+        if not self._no_render_thing:
+            if o.is_owl_nothing():
+                return _MAN_SYNTAX.BOTTOM
+            if o.is_owl_thing():
+                return _MAN_SYNTAX.TOP
+        return self._sfp(o)
 
     @render.register
     def _(self, p: OWLPropertyExpression) -> str:
@@ -303,7 +312,7 @@ class ManchesterOWLSyntaxOWLObjectRenderer(OWLObjectRenderer):
 
     @render.register
     def _(self, p: OWLObjectInverseOf) -> str:
-        return "%s(%s)" % (self.render(p.get_named_property()), _MAN_SYNTAX.INVERSE)
+        return "%s %s" % (_MAN_SYNTAX.INVERSE, self.render(p.get_named_property()))
 
     @render.register
     def _(self, r: OWLObjectMinCardinality) -> str:
@@ -322,7 +331,7 @@ class ManchesterOWLSyntaxOWLObjectRenderer(OWLObjectRenderer):
 
     @render.register
     def _(self, r: OWLObjectHasSelf) -> str:
-        return "%s %s %s" % (self.render(r.get_property()), _MAN_SYNTAX.EXISTS, _MAN_SYNTAX.SELF)
+        return "%s %s" % (self.render(r.get_property()), _MAN_SYNTAX.SELF)
 
     @render.register
     def _(self, r: OWLObjectHasValue):
